#' CLEAN AND CONSOLIDATE SEED HEAD DATA
#' 
#' This script loads seed head data from 2019, 2021, and 2023. Data is cleaned
#' and combined into a more comprehensive format to improve readability.
#' After running this script, metadata can be generated via SCRIPT_NAME


# Setup ------------------------

# Initialize here::here()
try(here::i_am("seed_mass_estimates/src/01_clean_seed_head_data.R"))

# Load required packages
library(here)
library(tidyverse)

# Use standardized folder structure throughout 
code_dir <- here("src")
data_dir <- here("data")
results_dir <- here("results")

# Load data from ServCat ------------------------
source(here(code_dir, "get_files_from_servcat.R"))

# Read and save 2019 data
data2019 <- get_file_from_servcat(250495) |>
  read_csv() |>
  # There's 32 duplicate rows in this data, decision to remove to clean data
  distinct()

write_csv(data2019, here(data_dir, "d2m_data_2019.csv"))

# Read and save 2021 data
data2021 <- get_file_from_servcat(250496) |>
  read_csv() |>
  mutate(
    year = 2021,
    emerged = f2t_length_mm / sh_length_mm
  ) |>
  # unclear if date is collected or measured so we are excluding
  # taxonID will be added back in later as species group instead of species
  dplyr::select(-date, -taxonID)

write_csv(data2021, here(data_dir, "d2m_data_2021.csv"))

# Read and save 2023 data
data2023 <- get_file_from_servcat(253218) |>
  read_csv() |>
  mutate(
    year = 2023,
    emerged = f2t_length_mm / sh_length_mm,
    # data was measured in grams, convert to mg
    mass_mg = mass_mg*1000
  ) |>
  group_by(bag_id) |>
  mutate(seedhead_id = row_number()) |>
  ungroup() |>
  # taxonID will be added back in later as species group instead of species
  dplyr::select(-taxonID)

write_csv(data2023, here(data_dir, "d2m_data_2023.csv"))

# Load and format raw data from Emilio -----------------------
# 2019 swamp timothy bag data analyzed with FIJI
# Moving forward, this dataset will not be used. Loading and cleaning the data
# is only to crosswalk to another dataset so these entries can be removed
swamp_timothy_bags_2019 <- read_csv(here(
  data_dir, "compiled_timothy_measurements05Jan2020.csv")) |>
  mutate(
    LIT = substr(photoID, 1, 3),
    LIT = case_when(
      LIT == "PXL" ~ "PIX",
      LIT == "STL" ~ "SLW",
      .default = as.character(LIT)
    ),
    year = 2019,
    vernacularName = "Swamp_Timothy"
  ) |>
  rename(
    sh_length_mm = Major,
    sh_width_mm = Minor,
    seedhead_id = sh.no,
    bag_id = photoID
  )
# Current problems with the data set as of May 5, 2024
# Some seed within a bag have identical measurements (possible but unlikely)

# 2019 swamp timothy set data analyzed with FIJI
swamp_timothy_sets_2019 <- readxl::read_excel(
  here(data_dir, "Ten_Seed_heads.xlsx"),
  sheet = "Sheet1") |>
  mutate(
    LIT = substr(id, 1, 3),
    LIT = case_when(
      LIT == "SL_" ~ "SLW",
      .default = as.character(LIT)
    ),
    year = 2019,
    vernacularName = "Swamp_Timothy"
  ) |>
  rename(
    sh_length_mm = Major,
    sh_width_mm = Minor,
    seedhead_id = seed.no,
    mass_mg = mass.mg,
    bag_id = id
  ) |>
  # Remove duplicates for Stone Lakes and Kern
  # SLW has duplicates -- SL_X_H_LARGE and SL_XQ_H_LARGE are identical
  # -- SL_XQ_H_SMALL and SL_X_H_SMALL are the same but with different mass
  # KRN has duplicates -- KRN_9_L_MEDIUM and KRN_9_L_LARGE are the same but
  # with different masses
  # PIX_1_H_SMALL has two seeds with identical measurements (possible but unlikely)
  filter(!str_detect(bag_id, 'SL_XQ_H_LARGE'),
  # In cases where duplicates have same dimensions but different mass, there is
  # no way to determine which is the correct value, so both must be removed.
  !str_detect(bag_id, 'SL_XQ_H_SMALL'),
  !str_detect(bag_id, 'SL_X_H_SMALL'),
  !str_detect(bag_id, 'KRN_9_L_MEDIUM'),
  !str_detect(bag_id, 'KRN_9_L_LARGE'),
         )

# Crosswalk data to get bag mass -----------------------

# Summarize the 2019 swamp timothy data for cross-walking to mass
ST_bags_crosswalk <- swamp_timothy_bags_2019 |>
  group_by(LIT, year, vernacularName, bag_id) |>
  # Get mean for each bag
  # Round numbers a little so that they will match later
  summarize(
    sh_length_mm = round(mean(sh_length_mm), 6),
    sh_width_mm = mean(sh_width_mm)
  )

ST_sets_crosswalk <- swamp_timothy_sets_2019 |>
  group_by(LIT, year, vernacularName, bag_id) |>
  summarize(
    sh_length_mm = round(mean(sh_length_mm), 6),
    sh_width_mm = mean(sh_width_mm),
    mass_mg = mean(mass_mg)/10
  )

ST_2019_mass_bags <- data2019 |>
  dplyr::filter(vernacularName == "Swamp_Timothy") |>
  dplyr::select(LIT, year, vernacularName, sh_length_mm, mass_mg) |>
  mutate(
    sh_length_mm = round(sh_length_mm, 6)) |>
  full_join(ST_bags_crosswalk,
            by = join_by(LIT, year, vernacularName, sh_length_mm))

ST_2019_mass_bags <- ST_2019_mass_bags[is.na(ST_2019_mass_bags$bag_id),] |>
  dplyr::select(LIT, year, vernacularName, sh_length_mm, mass_mg)

# Crosswalk to check data is as we expect
ST_2019_mass_bags_sets <- ST_2019_mass_bags |>
  right_join(ST_sets_crosswalk,
            by = join_by(
              LIT, year, vernacularName, sh_length_mm, mass_mg))
# Data is as expected, the ten seed head data can be used going forward

# Clean and consolidate 2019 data -----------------------

# Clean swamp timothy data
ST_2019_cleaned <- swamp_timothy_sets_2019 |>
  dplyr::select(LIT, year, vernacularName, bag_id, seedhead_id, mass_mg, 
                sh_length_mm, sh_width_mm) |>
  mutate(f2t_length_mm = NA,
         emerged = f2t_length_mm / sh_length_mm,
         bag_mass_mg = mass_mg,
         bag_seedhead_count = 10,
         mass_mg = bag_mass_mg/bag_seedhead_count)

# Clean watergrass data
data2019_watergrass <- data2019 |>
  dplyr::filter(vernacularName == "Watergrass") |>
  mutate(seedhead_id = 1,
         bag_mass_mg = mass_mg,
         bag_seedhead_count = 1) |>
  group_by(LIT) |>
  mutate(bag_id = paste0(LIT, "_", row_number())) |>
  ungroup()

# Combine swamp timothy and watergrass data
data2019_cleaned <- full_join(ST_2019_cleaned, data2019_watergrass) |>
  mutate(recorder = "EL",
         observer = "EL")

# Clean 2021 data -----------------------

data2021_cleaned <- data2021 |>
  group_by(bag_id) |>
  mutate(bag_seedhead_count = max(seedhead_id)) |>
  ungroup() |>
  mutate(bag_mass_mg = mass_mg,
         mass_mg = bag_mass_mg/bag_seedhead_count)

# Join 2019 and 2021 data -----------------------
data2019_2021_cleaned <- full_join(data2019_cleaned, data2021_cleaned) |>
  mutate(eventDate = NA,
         taxonID =dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ 41602,
           vernacularName == "Watergrass" ~ 40667,
           vernacularName == "Smartweed" ~ 823756
           ),
         fwsTaxonCode = dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ 141915,
           vernacularName == "Watergrass" ~ 144529,
           vernacularName == "Smartweed" ~ 884489
           ),
         scientificName = dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ "Crypsis",
           vernacularName == "Watergrass" ~ "Echinochloa",
           vernacularName == "Smartweed" ~ "Persicaria"
           )
  ) |>
  rename(bagID = bag_id,
         seedHeadID = seedhead_id,
         bagMass = bag_mass_mg,
         bagSeedHeadCount = bag_seedhead_count,
         recordedBy = recorder,
         observedBy = observer)

# Clean 2023 data -----------------------

data2023_cleaned <- data2023 |>
  group_by(bag_id) |>
  mutate(bag_seedhead_count = max(seedhead_id)) |>
  ungroup() |>
  mutate(bag_mass_mg = mass_mg,
         mass_mg = bag_mass_mg/bag_seedhead_count,
         bag_id = as.character(bag_id),
         taxonID = dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ 41605,
           vernacularName == "Watergrass" ~ 40667,
           vernacularName == "Smartweed" ~ 823756,
           vernacularName == "Foxtail_Timothy" ~ 41603
         ),
         fwsTaxonCode = dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ 141918,
           vernacularName == "Foxtail_Timothy" ~ 141916,
           vernacularName == "Watergrass" ~ 144529,
           vernacularName == "Smartweed" ~ 884489
         ),
         scientificName = dplyr::case_when(
           vernacularName == "Swamp_Timothy" ~ "Crypsis schoenoides",
           vernacularName == "Foxtail_Timothy" ~ "Crypsis alopecuroides",
           vernacularName == "Watergrass" ~ "Echinochloa",
           vernacularName == "Smartweed" ~ "Persicaria"
         )
  ) |>
  rename(bagID = bag_id,
         seedHeadID = seedhead_id,
         bagMass = bag_mass_mg,
         bagSeedHeadCount = bag_seedhead_count,
         recordedBy = recorder,
         observedBy = observer,
         eventDate = date_collected)

# Join 2019/21 and 2023 data -----------------------
all_seed_data_cleaned <- full_join(data2019_2021_cleaned, data2023_cleaned) |>
  mutate(
    institutionCode = "USFWS",
    datasetName = "Seed Head Data for Wetland Habitat Assessment Protocol Framework",
    datasetID = "https://ecos.fws.gov/ServCat/Reference/Profile/167449",
    basisOfRecord = "Specimen",
    eventID = paste(LIT, year, bagID, vernacularName, sep = "_"),
    occurrenceID = paste(eventID, seedHeadID, sep = "_"),
    decimalLatitude = dplyr::case_when(
      LIT == "SLW" ~ 38.37,
      LIT == "MDC" ~ 41.46,
      LIT == "SAL" ~ 33.18,
      LIT == "KRN" ~ 35.76,
      LIT == "PIX" ~ 35.92,
      LIT == "CLS" ~ 39.19,
      LIT == "SAC" ~ 39.43
    ),
    decimalLongitude = dplyr::case_when(
      LIT == "SLW" ~ -121.50,
      LIT == "MDC" ~ -120.52,
      LIT == "SAL" ~ -115.62,
      LIT == "KRN" ~ -119.58,
      LIT == "PIX" ~ -119.38,
      LIT == "CLS" ~ -122.04,
      LIT == "SAC" ~ -122.19
    ),
    geodeticDatum = "D WGS 1984"
    )

# Save data to CSV -----------------------

write_csv(all_seed_data_cleaned, here(results_dir, "whap_seedmodeldata_2019_2023.csv"))
