# Derivation of model for ST mass/m2 --------------
# Based on n_seed_head and avg_length_mm
# Emilio A. Laca and Julia Michaels

# Load necessary packages ------------
library(tidyverse)
library(lme4)
library(readxl)


# Read and prep ST data ----------
# ST quadrat data with dry seed head  mass from quadrat subsamples for lab photos
stqdt <- read.csv("Timothy_tracking05Jan2020.csv", stringsAsFactors = TRUE)

# ST data from photos where length of each seed head was measured in FIJI
stdat2 <- read.csv("compiled_timothy_measurements05Jan2020.csv",
                   stringsAsFactors = TRUE)

stdat2$spp <- factor("ST")

# Select records that contain seed head mass in stqdt
stqdt.seed <- stqdt %>%
  filter(smpl.type %in% c("Seed", "Seed(subsampled)"))

stqdt.seed$smpl.type <- droplevels(stqdt.seed$smpl.type)

# Calculate number and average seed head length by photo
stdat2.by.qdt <- stdat2 %>%
  group_by(photoID) %>%
  summarize(avg.length.mm = mean(Major, na.rm = TRUE),
            avg.area.mm2 = mean(Area, na.rm = TRUE),
            n.infl.photo = n())

# Join seed head mass with number and length
stqdt2.seed <- full_join(stqdt2.seed, stdat2.by.qdt)

# Read data from 10 similar seed heads per group picked by hand
Ten_Seed_heads <- read_excel("Ten_Seed_heads.xlsx")

# Separate the unique mass and id for each set of 10 seed heads
mass.10 <- unique(Ten_Seed_heads[, c("id", "mass.mg")])

#Calculate average characteristics of each group of 10 seed heads
length.10 <- Ten_Seed_heads %>%
  group_by(id) %>%
  summarise(length.mm = mean(Major),
            width.mm = mean(Minor),
            area.mm2 = mean(Area),
            perim.mm = mean(Perim.))

# Join average dimensions with mass of ten seed heads
ten_sh <- full_join(mass.10, length.10)

# Join relevant columns of stqdt2.seed and ten_sh
stpart1 <- stqdt2.seed %>%
  dplyr::select(id = photoID,
         mass.mg = dry.mass.g,
         avg.area.mm2 = avg.area.mm2,
         avg.length.mm = avg.length.mm,
         n.sh = n.infl.photo) %>%
  mutate(mass.mg = mass.mg * 1000)

stpart2 <- ten_sh %>%
  dplyr::select(id = id,
         mass.mg = mass.mg,
         avg.area.mm2 = area.mm2,
         avg.length.mm = length.mm) %>%
  mutate(no.sh = 10)

st.all <- rbind(stpart1, stpart2)

# Model seed head mass as a function of average length and number --------

# Remove one outlier with value outside the controlled 10 sh data
st.all.noO <- st.all[-3,]

saveRDS(st.all.noO, "st_all_noO.rds")

# Best model to predict mass based on length and no. seed heads
st.m1 <- lm(log(mass.mg) ~ log(avg.length.mm) + log(no.sh), st.all.noO)

# Save model object -----
# For prediction, newdata file should have:
# seed head length in mm,
# number of seed heads per m2.
# Predict as follows to get kg/ha:

# mass.kg.ha = exp(predict(m2, newdata = .)) / 100

# Save model object in current directory for future use ------
saveRDS(st.m1, "ST_seed_head_mass_model.rds")



# Test for effect of refuge (Jan 2021) ----

st.all.noO.b <- st.all.noO %>%
  mutate(refuge = factor(substr(id, 1, 3)))

st.m1b <- lm(log(mass.mg) ~ log(avg.length.mm) * log(no.sh) * refuge,
             data = st.all.noO.b,
             na.action = na.exclude)

anova(st.m1b)
summary(st.m1b)

# Compare models with repeated cross-validation -----

# Define training control
set.seed(123)

train.control <- trainControl(method = "repeatedcv",
                              number = 10, repeats = 10)

# Train the model
modelA <- train(log(mass.mg) ~ log(avg.length.mm) + log(no.sh),
                data = st.all.noO.b,
                method = "lm",
                trControl = train.control)

# Summarize the results
print(modelA)

# Train the model
modelB <- train(log(mass.mg) ~ log(avg.length.mm) * log(no.sh) * refuge,
                data = st.all.noO.b,
                method = "lm",
                trControl = train.control)

# Summarize the results
print(modelB)



plot(fitted(st.m1 ) ~ log(mass.mg), data = st.all.noO)
points(fitted(st.m1b ) ~ log(mass.mg), data = st.all.noO.b, col = "red")
