# Derivation of model for WG mass per unit area --------------
# Based on n_seed_head and avg_length_mm
# Emilio A. Laca and Julia Michaels

# Load necessary packages ------------
library(tidyverse)
library(lme4)
library(car)
library(mgcv)
library(caret)


# Read and prep ST data ----------
# WG seed head fresh mass was measured for each individual seed head;
# then, all seed heads in each quadrat were put in a single bag and dried.
# In order to get the dry mass of each seed head, we obtain the proportion
# of dry mass for each bag and multiply by the fresh mass of individual
# seed heads.

# WG quadrat data with dry seed head mass per quadrat
wgqdt.seed <- read.csv("Watergrass_tracking.csv",
                       stringsAsFactors = TRUE) %>%
  mutate(qid = paste(refuge, spp, density, quadrat, sep = "")) %>%
  filter(smpl.type != "Biomass" & !is.na(dry.mass.g))

# Read WG individual seed head length and fresh mass obtained by hand in lab
wgdat <- read.csv('Watergrass_Measurements19Nov2019.csv',
                  stringsAsFactors = TRUE) %>%
  mutate(qid = paste(refuge, spp, density, quadrat, sep = "")) %>%
  filter(!is.na(infl.fmass.g) & !is.na(F2T.cm))

# Calculate total fresh mass of seed heads per quadrat and put into wgqdt
# to calculate proportion of dry matter.
fresh.mass <- wgdat %>%
  group_by(qid) %>%
  summarise(tot.infl.fmass.g = sum(infl.fmass.g))

# Join fresh mass with dry mass data and calculate proportion dry matter
# when proportion of DM is gretaer than 1 it is set to 1
wgqdt.seed <- full_join(wgqdt.seed, fresh.mass) %>%
  mutate(p.dm = ifelse(dry.mass.g / tot.infl.fmass.g > 1,
                       1, dry.mass.g / tot.infl.fmass.g))


# Add quadrat size and proportion DM to individual seed head data
# Remove seed heads that had dropped seeds (Pixley) and a couple of outliers
wgdat <- left_join(wgdat, wgqdt.seed[, c("qid", "p.dm", "qdt.m2")]) %>%
  mutate(mass.per.sh.mg = 1000 * infl.fmass.g * p.dm,
         emerged = F2T.cm / sh.len.cm) %>%
  filter(mass.per.sh.mg < 1000 &
           mass.per.sh.mg > 5 &
           sh.len.cm < 22 &
           refuge != "Pixley") %>%
  arrange(refuge, quadrat, mass.per.sh.mg)

# Model seed head mass as a function of individual length, etc.--------
wg.m1 <- lm(log(mass_per_sh_mg  + 20) ~ sh_len_cm +
              emerged +
              F2T_cm,
            data = wgdat,
            na.action = na.exclude)

saveRDS(wgdat, "wgdat.rds")

# Save model object -----
# For prediction, new_data file should have:
# seed head length in cm, column named "sh.len.cm"
# flag leaf collar to tip length in cm named "F2T.cm"
# ratio (F2T.cm - sh.len.cm) / (F2T.cm + sh.len.cm) named "emerged"
# Predict as follows to get mass per seedhead in mg:

# mass.per.sh.mg = exp(predict(wg.m1, newdata = new_data)) - 15

# Save model object in current directory for future use ------
saveRDS(wg.m1, "WG_seed_head_mass_model.rds") # model for 2019 on


# Model seed head mass as a function of individual length ONLY; to use with 2018quadrat data --------
wg.m2 <- lm(log(mass.per.sh.mg  + 20) ~ sh.len.cm,
            data = wgdat,
            na.action = na.exclude)

saveRDS(wg.m2, "WG_seed_head_mass_modelB.rds") # model for 2018 only


# Test for effect of refuge (Jan 2021) ----

wg.m1b <- lm(log(mass_per_sh_mg  + 20) ~ (sh_len_cm +
                                            emerged +
                                            F2T_cm) * refuge,
             data = wgdat,
             na.action = na.exclude)

anova(wg.m1b)
summary(wg.m1b)

wg.m1c <- lm(log(mass_per_sh_mg  + 20) ~ (sh_len_cm +
                                           emerged +
                                            F2T_cm)^2 * refuge,
             data = wgdat,
             na.action = na.exclude)

anova(wg.m1c)
summary(wg.m1c)

wg.m1d <- gam(log(mass_per_sh_mg  + 20) ~ s(sh_len_cm,
                                            emerged,
                                            F2T_cm,
                                            k = 20,
                                            by = refuge),
              data = wgdat,
              na.action = na.exclude)

gam.check(wg.m1d)
anova(wg.m1d)
summary(wg.m1d)
plot(wg.m1d)
vis.gam(wg.m1d, view = c("sh_len_cm", "emerged"))
vis.gam(wg.m1d, view = c("sh_len_cm", "F2T_cm"))

new_data <- wgdat[sample(1:dim(wgdat)[1], 10), ]

predict(wg.m1, newdata = new_data, interval = "prediction", se.fit = TRUE)
predict(wg.m1, newdata = new_data, interval = "confidence", se.fit = TRUE)
predict(wg.m1d, newdata = new_data, interval = "prediction", se.fit = TRUE)

wgdat2 <- wgdat %>%
  mutate(p_wg.m1 = predict(wg.m1, se.fit = TRUE)$fit,
         p_wg.m1b = predict(wg.m1b, se.fit = TRUE)$fit,
         p_wg.m1c = predict(wg.m1c, se.fit = TRUE)$fit,
         p_wg.m1d = predict(wg.m1d, se.fit = TRUE)$fit,
         se_wg.m1 = predict(wg.m1, se.fit = TRUE)$se.fit,
         se_wg.m1b = predict(wg.m1b, se.fit = TRUE)$se.fit,
         se_wg.m1c = predict(wg.m1c, se.fit = TRUE)$se.fit,
         se_wg.m1d = predict(wg.m1d, se.fit = TRUE)$se.fit)

scatterplot(p_wg.m1 ~ p_wg.m1b | refuge,
            data = wgdat2,
            smooth = FALSE)

abline(0, 1, col = "red")

# Prediction se is smaller with wg.m1 than wg.m1b for all refuges -----

scatterplot(se_wg.m1 ~ se_wg.m1b | refuge,
            data = wgdat2,
            smooth = FALSE)

abline(0, 1, col = "red")

plot(log(mass_per_sh_mg  + 20) ~ p_wg.m1,
     data = wgdat2,
     ylab = "Observed mass (log(mass) + 20)",
     xlab = "Predicted mass")

points(log(mass_per_sh_mg  + 20) ~ p_wg.m1c,
       data = wgdat2,
       col = "red")

points(p_wg.m1c ~ log(mass_per_sh_mg  + 20),
       col = "red",
       data = wgdat2)

abline(0, 1)

scatterplot(residuals(wg.m1) ~ residuals(wg.m1b) | refuge,
            data = wgdat2,
            smooth = FALSE)

scatterplot(residuals(wg.m1) ~ residuals(wg.m1d) | refuge,
            data = wgdat2,
            smooth = FALSE)

scatterplot(p_wg.m1b ~ log(mass_per_sh_mg  + 20) | refuge,
     data = wgdat2,
     smooth = FALSE)

scatterplot(residuals(wg.m1) ~ residuals(wg.m1d) | refuge,
            data = wgdat2,
            smooth = FALSE)

scatterplot(se_wg.m1d ~ se_wg.m1 | refuge,
            data = wgdat2,
            smooth = FALSE)


# Compare models with repeated cross-validation -----

# Define training control
set.seed(123)

train.control <- trainControl(method = "repeatedcv",
                              number = 10, repeats = 10)

# Train the model
model1 <- train(log(mass_per_sh_mg  + 20) ~ sh_len_cm +
                 emerged +
                 F2T_cm,
               data = wgdat, method = "lm",
               trControl = train.control)

# Summarize the results
print(model1)

# Train the model
model1c <- train(log(mass_per_sh_mg  + 20) ~ (sh_len_cm +
                                              emerged +
                                              F2T_cm)^2 * refuge,
               data = wgdat, method = "lm",
               trControl = train.control)

# Summarize the results
print(model1c)



